#include <config.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <gconf/gconf-client.h>
#include <glade/glade.h>
#include <libgnome/gnome-i18n.h>

typedef struct
{
  gchar *lock_dialog_theme;
  GConfClient *gconf_client;
  GtkListStore *list_store;
  GtkTreeSelection *theme_tree_selection;
  GtkCheckButton *lock_enabled_check;
  GtkHScale *lock_delay_scale;
  GtkCheckButton *user_switch_enabled_check;
  GtkCheckButton *logout_enabled_check;
  GtkHScale *logout_delay_scale;
} MainData;

static gboolean command_line_get = FALSE;
static gchar *command_line_set = NULL;
static GOptionEntry entries[] = {
  {"show", 's', 0, G_OPTION_ARG_NONE, &command_line_get,
   "Show the current lock dialog theme", NULL},
  {"change", 'c', 0, G_OPTION_ARG_STRING, &command_line_set,
   "Change the lock dialog theme", "THEME"},
};



void
parse_args (gint * argc, gchar *** argv)
{
  GError *error = NULL;
  GOptionContext *context;
  context = g_option_context_new (" - GNOME Lock Dialog Preferences");
  g_option_context_add_main_entries (context, entries, NULL);
  g_option_context_add_group (context, gtk_get_option_group (FALSE));
  if (!g_option_context_parse (context, argc, argv, &error))
    {
      g_error ("%s\n", error->message);
    }
}

void
check_lock_dialog_themes (GFunc run_on_each, gpointer user_data)
{
  GDir *screensaver_dir;
  GError *error;
  gchar *filename;
  gchar *theme_name;
  screensaver_dir = g_dir_open ("/usr/share/gnome-screensaver", 0, &error);
  filename = (gchar *) g_dir_read_name (screensaver_dir);
  while (filename != NULL)
    {
      if (g_str_has_prefix (filename, "lock-dialog-") == TRUE &&
	  g_str_has_suffix (filename, ".gtkrc") == FALSE)
	{
	  theme_name = g_strndup (filename + 12, g_utf8_strlen (filename + 12, -1) - 6);
	  run_on_each (theme_name, user_data);
	  g_free (theme_name);
	}
      filename = (gchar *) g_dir_read_name (screensaver_dir);
    }
  g_dir_close (screensaver_dir);
}

void
list_store_fill (gchar * theme_name, gpointer user_data)
{
  MainData *main_data = user_data;
  GtkTreeIter iter;
  gtk_list_store_append (main_data->list_store, &iter);
  gtk_list_store_set (main_data->list_store, &iter, 0, theme_name, -1);
  if (!g_strcmp0 (theme_name, main_data->lock_dialog_theme))
    {
      gtk_tree_selection_select_iter (GTK_TREE_SELECTION
				      (main_data->theme_tree_selection),
				      &iter);
    }
}

void
lock_theme_selection_changed (GtkTreeSelection * selection,
			      gpointer user_data)
{
  MainData *main_data = user_data;
  GtkTreeModel *model;
  GtkTreeIter iter;
  gchar *new_theme;
  if (gtk_tree_selection_get_selected (selection, &model, &iter))
    {
      gtk_tree_model_get (model, &iter, 0, &new_theme, -1);
      gconf_client_set_string (main_data->gconf_client,
			       "/apps/gnome-screensaver/lock_dialog_theme",
			       new_theme, NULL);
      g_free (new_theme);
    }
  return;
}

void
lock_enabled_check_toggled (GtkCheckButton * check, gpointer user_data)
{
  MainData *main_data = user_data;
  gboolean new_value =
    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
                               (main_data->lock_enabled_check));
    gconf_client_set_bool (main_data->gconf_client,
			   "/apps/gnome-screensaver/lock_enabled", new_value, NULL);
}

void
lock_delay_scale_value_changed (GtkHScale * hscale, gpointer user_data)
{
  MainData *main_data = user_data;
  gint new_value =
    gtk_range_get_value (GTK_RANGE
                               (hscale));
    gconf_client_set_int (main_data->gconf_client,
			   "/apps/gnome-screensaver/lock_delay", new_value, NULL);
}

void
user_switch_enabled_check_toggled (GtkCheckButton * check, gpointer user_data)
{
  MainData *main_data = user_data;
  gboolean new_value =
    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
                               (check));
    gconf_client_set_bool (main_data->gconf_client,
			   "/apps/gnome-screensaver/user_switch_enabled", new_value, NULL);
}

void
logout_enabled_check_toggled (GtkCheckButton * check, gpointer user_data)
{
  MainData *main_data = user_data;
  gboolean new_value =
    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
                               (check));
    gconf_client_set_bool (main_data->gconf_client,
			   "/apps/gnome-screensaver/logout_enabled", new_value, NULL);
}

void
logout_delay_scale_value_changed (GtkHScale * scale, gpointer user_data)
{
  MainData *main_data = user_data;
  gint new_value =
    gtk_range_get_value (GTK_RANGE
                               (scale));
    gconf_client_set_int (main_data->gconf_client,
			   "/apps/gnome-screensaver/logout_delay", new_value, NULL);
}

void
lock_dialog_theme_gconf_callback (GConfClient * client,
				  guint cnxn_id,
				  GConfEntry * entry, gpointer user_data)
{
  MainData *main_data = user_data;
  if (gconf_entry_get_value (entry) != NULL &&
      gconf_entry_get_value (entry)->type == GCONF_VALUE_STRING)
    {
      gchar *new_value =
	(gchar *) gconf_value_get_string (gconf_entry_get_value (entry));
      if (g_strcmp0 (new_value, main_data->lock_dialog_theme))
        {
          //TODO
        }
    }
}

void
lock_enabled_gconf_callback (GConfClient * client,
			     guint cnxn_id,
			     GConfEntry * entry, gpointer user_data)
{
  MainData *main_data = user_data;
  if (gconf_entry_get_value (entry) != NULL &&
      gconf_entry_get_value (entry)->type == GCONF_VALUE_BOOL)
    {
      gboolean new_value =
	gconf_value_get_bool (gconf_entry_get_value (entry));
      gboolean check_state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				   (main_data->lock_enabled_check));
      if (check_state != new_value)
        {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				   (main_data->lock_enabled_check),
				   new_value);
	}
    }
}

void
lock_delay_gconf_callback (GConfClient * client,
			   guint cnxn_id,
			   GConfEntry * entry, gpointer user_data)
{
  MainData *main_data = user_data;
  if (gconf_entry_get_value (entry) != NULL &&
      gconf_entry_get_value (entry)->type == GCONF_VALUE_INT)
    {
      gint new_value =
	gconf_value_get_int (gconf_entry_get_value (entry));
      gint check_state = gtk_range_get_value (GTK_RANGE
				   (main_data->lock_delay_scale));
      if (check_state != new_value)
        {
      gtk_range_set_value (GTK_RANGE (main_data->lock_delay_scale),
				   new_value);
	}
    }
}

void
user_switch_enabled_gconf_callback (GConfClient * client,
				    guint cnxn_id,
				    GConfEntry * entry, gpointer user_data)
{
  MainData *main_data = user_data;
  if (gconf_entry_get_value (entry) != NULL &&
      gconf_entry_get_value (entry)->type == GCONF_VALUE_BOOL)
    {
      gboolean new_value =
	gconf_value_get_bool (gconf_entry_get_value (entry));
      gboolean check_state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				   (main_data->lock_enabled_check));
      if (check_state != new_value)
        {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				   (main_data->user_switch_enabled_check),
				   new_value);
	}
    }
}

void
logout_enabled_gconf_callback (GConfClient * client,
			       guint cnxn_id,
			       GConfEntry * entry, gpointer user_data)
{
  MainData *main_data = user_data;
  if (gconf_entry_get_value (entry) != NULL &&
      gconf_entry_get_value (entry)->type == GCONF_VALUE_BOOL)
    {
      gboolean new_value =
	gconf_value_get_bool (gconf_entry_get_value (entry));
      gboolean check_state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				   (main_data->lock_enabled_check));
      if (check_state != new_value)
        {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				   (main_data->logout_enabled_check),
				   new_value);
	}
    }
}

void
logout_delay_gconf_callback (GConfClient * client,
			     guint cnxn_id,
			     GConfEntry * entry, gpointer user_data)
{
  MainData *main_data = user_data;
  if (gconf_entry_get_value (entry) != NULL &&
      gconf_entry_get_value (entry)->type == GCONF_VALUE_INT)
    {
      gint new_value =
	gconf_value_get_int (gconf_entry_get_value (entry));
      gint check_state = gtk_range_get_value (GTK_RANGE
				   (main_data->logout_delay_scale));
      if (check_state != new_value)
        {
        gtk_range_set_value (GTK_RANGE (main_data->logout_delay_scale),
				   new_value);
	}
    }
}

static void
on_destroy (GtkWidget * widget, gpointer data)
{
  gtk_main_quit ();
}

int
main (int argc, char *argv[])
{
  MainData main_data;
  GladeXML *glade_xml;
  GtkDialog *main_dialog;
  GtkTreeView *theme_tree;
  GtkTreeViewColumn *theme_tree_column;
  GtkCellRenderer *theme_tree_cell;
  gboolean lock_enabled = FALSE;
  gint lock_delay = 120;
  gboolean user_switch_enabled = FALSE;
  gboolean logout_enabled = FALSE;
  gint logout_delay = 0;

  bindtextdomain (PACKAGE, LOCALEDIR);
  bind_textdomain_codeset (PACKAGE, "UTF-8");
  textdomain (PACKAGE);

  parse_args (&argc, &argv);

  main_data.gconf_client = gconf_client_get_default ();

  gconf_client_add_dir (main_data.gconf_client, "/apps/gnome-screensaver",
			GCONF_CLIENT_PRELOAD_NONE, NULL);

  main_data.lock_dialog_theme =
    gconf_client_get_string (main_data.gconf_client,
			     "/apps/gnome-screensaver/lock_dialog_theme",
			     NULL);
  lock_enabled =
    gconf_client_get_bool (main_data.gconf_client,
			   "/apps/gnome-screensaver/lock_enabled", NULL);
  lock_delay =
    gconf_client_get_int (main_data.gconf_client,
			  "/apps/gnome-screensaver/lock_delay", NULL);
  user_switch_enabled =
    gconf_client_get_bool (main_data.gconf_client,
			   "/apps/gnome-screensaver/user_switch_enabled",
			   NULL);
  logout_enabled =
    gconf_client_get_bool (main_data.gconf_client,
			   "/apps/gnome-screensaver/logout_enabled", NULL);
  logout_delay =
    gconf_client_get_int (main_data.gconf_client,
			  "/apps/gnome-screensaver/logout_delay", NULL);

  if (command_line_get == TRUE)
    {
      g_print ("%s\n", main_data.lock_dialog_theme);
      return (0);
    }
  if (command_line_set != NULL)
    {
      return (0);
    }

  gtk_init (&argc, &argv);

  glade_xml =
    glade_xml_new (GLADE_DIR "/glockdialog-prefs.glade", NULL, NULL);

  main_data.list_store = gtk_list_store_new (1, G_TYPE_STRING);

  main_dialog = GTK_DIALOG (glade_xml_get_widget (glade_xml, "main_dialog"));
  theme_tree = GTK_TREE_VIEW (glade_xml_get_widget (glade_xml, "theme_tree"));
  main_data.lock_enabled_check =
    GTK_CHECK_BUTTON (glade_xml_get_widget (glade_xml, "lock_enabled_check"));
  main_data.lock_delay_scale =
    GTK_HSCALE (glade_xml_get_widget (glade_xml, "lock_delay_scale"));
  main_data.user_switch_enabled_check =
    GTK_CHECK_BUTTON (glade_xml_get_widget
		      (glade_xml, "user_switch_enabled_check"));
  main_data.logout_enabled_check =
    GTK_CHECK_BUTTON (glade_xml_get_widget (glade_xml, "logout_enabled_check"));
  main_data.logout_delay_scale =
    GTK_HSCALE (glade_xml_get_widget (glade_xml, "logout_delay_scale"));
  theme_tree_cell = gtk_cell_renderer_text_new ();
  theme_tree_column =
    gtk_tree_view_column_new_with_attributes ("Name",
					      GTK_CELL_RENDERER
					      (theme_tree_cell), "text", 0,
					      NULL);
  main_data.theme_tree_selection =
    gtk_tree_view_get_selection (GTK_TREE_VIEW (theme_tree));

  gtk_tree_selection_set_mode (main_data.theme_tree_selection,
			       GTK_SELECTION_SINGLE);
  gtk_tree_view_append_column (GTK_TREE_VIEW (theme_tree), theme_tree_column);

  gtk_tree_view_set_model (GTK_TREE_VIEW (theme_tree),
			   GTK_TREE_MODEL (main_data.list_store));

  check_lock_dialog_themes ((GFunc) * list_store_fill, &main_data);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
			       (main_data.lock_enabled_check), lock_enabled);
  gtk_range_set_value (GTK_RANGE (main_data.lock_delay_scale), lock_delay);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
			       (main_data.user_switch_enabled_check),
			       user_switch_enabled);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
			       (main_data.logout_enabled_check),
			       logout_enabled);
  gtk_range_set_value (GTK_RANGE (main_data.logout_delay_scale),
		       logout_delay);

  gconf_client_notify_add (main_data.gconf_client,
			   "/apps/gnome-screensaver/lock_dialog_theme",
			   lock_dialog_theme_gconf_callback, &main_data, NULL,
			   NULL);
  gconf_client_notify_add (main_data.gconf_client,
			   "/apps/gnome-screensaver/lock_enabled",
			   lock_enabled_gconf_callback, &main_data, NULL,
			   NULL);
  gconf_client_notify_add (main_data.gconf_client,
			   "/apps/gnome-screensaver/lock_delay",
			   lock_delay_gconf_callback, &main_data, NULL, NULL);
  gconf_client_notify_add (main_data.gconf_client,
			   "/apps/gnome-screensaver/user_switch_enabled",
			   user_switch_enabled_gconf_callback, &main_data,
			   NULL, NULL);
  gconf_client_notify_add (main_data.gconf_client,
			   "/apps/gnome-screensaver/logout_enabled",
			   logout_enabled_gconf_callback, &main_data, NULL,
			   NULL);
  gconf_client_notify_add (main_data.gconf_client,
			   "/apps/gnome-screensaver/logout_delay",
			   logout_delay_gconf_callback, &main_data, NULL,
			   NULL);


  glade_xml_signal_connect (glade_xml, "on_destroy", G_CALLBACK (on_destroy));
  g_signal_connect (G_OBJECT (main_data.theme_tree_selection), "changed",
		    G_CALLBACK (lock_theme_selection_changed), &main_data);
  g_signal_connect (G_OBJECT (main_data.lock_enabled_check), "toggled",
		    G_CALLBACK (lock_enabled_check_toggled), &main_data);
  g_signal_connect (G_OBJECT (main_data.lock_delay_scale), "value_changed",
		    G_CALLBACK (lock_delay_scale_value_changed), &main_data);
  g_signal_connect (G_OBJECT (main_data.user_switch_enabled_check), "toggled",
		    G_CALLBACK (user_switch_enabled_check_toggled), &main_data);
  g_signal_connect (G_OBJECT (main_data.logout_enabled_check), "toggled",
		    G_CALLBACK (logout_enabled_check_toggled), &main_data);
  g_signal_connect (G_OBJECT (main_data.logout_delay_scale), "value_changed",
		    G_CALLBACK (logout_delay_scale_value_changed),
		    &main_data);

  gtk_main ();

  return 0;
}
